/*
 * PowerGraph.pde 
 *           - Displays Estimated MSP430 & CC3000 Power consumption
 *             based on throughput
 * 
 * Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the project's author nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

class classPowerGraph {
  int ThumbX, ThumbY;
  int MyX, MyY;
  PImage axis, axisMask;
  int GraphThumbWidth = 150;
  int GraphThumbHeight = 50;
  int GraphWidth  = 480;
  int GraphHeight = 100;
  PFont f;
  
  classPowerGraph(int mx, int my) {
    ThumbX = mx;
    ThumbY = my;    
  }
  
  void setup() {
  }
  
  void draw(float[] InputBuffer, int DataLen, int mx, int my) {
    f = loadFont("ComicSansMS-48.vlw"); 
    MyX = mx;
    MyY = my; 
    // Scale to 750mW 
    int scaleThumbFactor = int(float(ThumbY)/0.75);
    int scaleFactor = int(float(100)/0.75);
    
    //
    // Draw Fixed Thumbnail
    //
    //draw data    
    beginShape();
    // draw Axis
    stroke(128);    
    line(ThumbX,ThumbY,ThumbX+GraphThumbWidth,ThumbY);
    line(ThumbX,ThumbY,ThumbX,ThumbY-GraphThumbHeight);
    for(int i = 1; i<(DataLen); i++)
    {     
      curve(ThumbX+int(i-1),  ThumbY - (InputBuffer[i-1]*scaleThumbFactor), ThumbX+int(i),  ThumbY - (InputBuffer[i]*scaleThumbFactor), ThumbX+int(i-1),  ThumbY - (InputBuffer[i-1]*scaleThumbFactor), ThumbX+int(i),  ThumbY - (InputBuffer[i]*scaleThumbFactor));
    }
    endShape();
    
    textFont(f,12); 
    text("0mW", ThumbX-30, ThumbY+15);
    text("750mW", ThumbX-60, ThumbY-35);
    textFont(f,16);
    
    //
    // Draw Main Graph
    //
    //draw data    
    beginShape();
    // draw Axis
    stroke(128);    
    line(MyX,MyY,MyX+GraphWidth,MyY);
    line(MyX,MyY,MyX,MyY-GraphHeight);
    for(int i = 1; i<(DataLen); i++)
    {     
      curve(MyX+int((i-1)*(3.2)),  MyY - (InputBuffer[i-1]*scaleFactor), MyX+int((i)*(3.2)),  MyY - (InputBuffer[i]*scaleFactor), MyX+int((i-1)*(3.2)),  MyY - (InputBuffer[i-1]*scaleFactor), MyX+int((i)*(3.2)),  MyY - (InputBuffer[i]*scaleFactor));
    }
    endShape();
    
    // Draw Coords
    text("0mW", MyX-40, MyY+15);
    text("750mW", MyX-75, MyY-80);
    //draw data
    //drawCursorValue(InputBuffer, DataLen);
  }
  
  void drawCursorValue(float[] InputBuffer, int Len){  
    if(mouseX > MyX && mouseX < (MyX+(1440/3)) &&
       mouseY > MyY && mouseY < (MyY+100))
    {      
      fill(30);
      rect(mouseX, MyY-30, 50,30);
      fill(255);
      // Convert Temperature to String
      String Power;
      Power = str(InputBuffer[int((mouseX-MyX)/Len)]);
      
      // Limit resolution to 0.1
      //Power = Power.substring(0,DecimalPlace+2);
      // Print temperature and units
      text(Power + "mW", mouseX, MyY-15);
      
      /*int time;
      time = int(GRAPH_WIDTH/Len) - int((mouseX-MyX)/Len);
      
      if(NumOfDataPoints > 60)
      {
        int hour = time / 60;
        int min = time % 60;
        String MinStr = str(min);
        // Pad with 0
        if(min < 10)
        {
          MinStr = "0" + str(min);
        }        
        text(str(hour) + ":" + MinStr, mouseX, MyY);
      }
      else
      {        
        time--;
        int sec = time % 60; 
        // Display Seconds
        text(str(sec) + "s", mouseX, MyY);
      }      */
    }
  }  
}
