/*******************************************************************************
 *
 *  Graph.pde 
 *           - Displays data with a gradient graph with a reflection.
 *             
 * 
 * Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the project's author nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
 
classGraph MyGraph;
byte[] TestBuffer;

class classGraph {
  int MyX, MyY;
  int NumOfDataPoints;
  float DataPerPixel;
  int GRAPH_WIDTH = 1440/3;
  PImage img = createImage(GRAPH_WIDTH, 100, RGB);
  PImage axis, axisMask;
  int UNITS_C = 0;
  int UNITS_F = 1;
  int units = UNITS_C;  
  
  classGraph(int mx, int my, int BufferSize, float Resolution) {
    MyX = mx;
    MyY = my;    
    NumOfDataPoints = BufferSize;
    DataPerPixel = Resolution;
  }
  
  void setup() {
    // Load axis image based on short-term or long-term graph
    if(NumOfDataPoints == 60)
    {
      axis = loadImage("Graph with 6 X Ticks.png");
    }
    else
    {
      axis = loadImage("Graph with 4 X Ticks.png");
    }
    axisMask = loadImage("axisMask2.png");
    img.loadPixels();
  }
  
  void draw(float[] InputBuffer,int mx, int my) {
    MyX = mx;
    MyY = my; 
    float[] ScaledBuffer = new float[1440];
    
    //draw data
    PImage reflection = createImage(GRAPH_WIDTH, 100, ARGB);       
    reflection.loadPixels();
    
    // Get Data Limits
    float DataSetMin = min(InputBuffer);
    float DataSetMax = max(InputBuffer);
    // Limit range
    if( units == UNITS_C )
    {
      DataSetMin = max(0, DataSetMin);
      DataSetMax = min(45, DataSetMax);
    }
    else
    {
      DataSetMin = max(0, DataSetMin);
      DataSetMax = min(100, DataSetMax);
    }
    // Ensure there is a delta
    if((DataSetMax - DataSetMin) < 4)
    {
      DataSetMax+=5;
      DataSetMin--;
    }
    
    //Scale Data Set for dynamic scaleing
    float delta;
    delta = max(1, DataSetMax - DataSetMin);
    float ScaleFactor = 100.0 / delta;
    
    for(int i = 0; i < InputBuffer.length; i++)
    {
      ScaledBuffer[i] = (InputBuffer[i] - (DataSetMin-2)) * ScaleFactor;
    }
    
    // Is Short or Long term data?  Buffer size is 60 for short-term
    if(InputBuffer.length > 60)
    {
      int samples = 3;
      for(int i = 0; i<1440; i=i+samples)
      {         
         // Avg 10 samples       
         float avg = 0;
         for(int s=0; s<samples;s++)
         {
           avg += ScaledBuffer[i+s];
         }   
         avg = int(avg / samples);
         
         reflection.loadPixels();       
         for(int c = 0; c < 100; c++)
         {
           color cl;
           if(c < avg)
           {
            // r,g,b
             cl = color(255, 112 - int(c*1.12), 35 - int(c*0.35));  
             //set(MyX+(i/samples)+5,MyY-c,cl);
             //cl = color(255, 112 - int(c*1.12), 35 - int(c*0.35), c*2); 
             reflection.pixels[(c*reflection.width)+(i/samples)] = cl; 
           }
           else
           {
             cl = color(0); 
             reflection.pixels[(c*reflection.width)+(i/samples)] = 0x262626;
           }            
         }     
      }
    }
    else
    {
      int samples = 8;
      for(int i = 0; i<GRAPH_WIDTH; i++)
      {                    
         for(int c = 0; c < 100; c++)
         {
           color cl;
           if(c < int(ScaledBuffer[i/samples]))
           {
            // r,g,b
             cl = color(255, 112 - int(c*1.12), 35 - int(c*0.35));  
             //set(MyX+(i/samples)+5,MyY-c,cl);
             //cl = color(255, 112 - int(c*1.12), 35 - int(c*0.35), c*2); 
             reflection.pixels[(c*reflection.width)+(i)] = cl; 
           }
           else
           {
             reflection.pixels[(c*reflection.width)+(i)] = 0x262626;
           }            
         }     
      }
    }
        
    
    reflection.updatePixels();
    
    // Draw upper
    pushMatrix(); 
    scale(1,-1);
    image(reflection, MyX, -(MyY+100));
    popMatrix();
    // draw axis
    image(axis, MyX-20, MyY-10); 
    
    // draw reflection    
    pushMatrix();      
    translate(MyX, MyY+100, 0); 
    reflection.mask(axisMask);
    image(reflection, 0, 20);
    popMatrix();
    
    drawScaleValues(DataSetMin, DataSetMax);
    drawCursorValue(InputBuffer);
  }
  
  void drawScaleValues(float DataSetMin, float DataSetMax) {
    if(NumOfDataPoints == 60)
    {
      text("60s", MyX-35, MyY+120);
      text("0S",  MyX+485, MyY+120);
    }
    else
    {
      text("24H", MyX-35, MyY+120);
      text("0H", MyX+485, MyY+120);
    }
    
    text(int(DataSetMin), MyX-35, MyY+100);
    text(int(DataSetMax), MyX-35, MyY);
  }
  
  // Draws the actual data value that the mouse is hovering over.
  void drawCursorValue(float[] InputBuffer){  
    if(mouseX > MyX && mouseX < (MyX+(GRAPH_WIDTH)) &&
       mouseY > MyY && mouseY < (MyY+100))
    {      
      fill(30);
      rect(mouseX, MyY-30, 50,30);
      fill(255);
      // Convert Temperature to String
      String temperature;
      temperature = str(InputBuffer[int((mouseX-MyX)/DataPerPixel)]);
      // Find the decimal location
      int DecimalPlace;
      DecimalPlace = temperature.indexOf(".");    
      // Limit resolution to 0.1
      temperature = temperature.substring(0,DecimalPlace+2);
      // Print temperature and units
      if( units == UNITS_C )
      {
        text(temperature + "C", mouseX, MyY-15);
      }
      else
      {
        text(temperature + "F", mouseX, MyY-15);
      }
      int time;
      time = int(GRAPH_WIDTH/DataPerPixel) - int((mouseX-MyX)/DataPerPixel);
      
      if(NumOfDataPoints > 60)
      {
        int hour = time / 60;
        int min = time % 60;
        String MinStr = str(min);
        // Pad with 0
        if(min < 10)
        {
          MinStr = "0" + str(min);
        }        
        text(str(hour) + ":" + MinStr, mouseX, MyY);
      }
      else
      {        
        time--;
        int sec = time % 60; 
        // Display Seconds
        text(str(sec) + "s", mouseX, MyY);
      }      
    }
  }
  
  void SetUnits(int val)
  {
    units = val;
  }
}
