/*
 * adc.c - DataLogger_prj.
 * 
 * Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the project's author nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#include "msp430fr5739.h"
#include "adc.h"
// 8-bit ADC conversion result array
volatile unsigned short ADC_Result[SIZE_OF_SEQUENCE];                

AdcMeasurement_t AdcDevices;

void AdcInit (void)
{
  //
  // Accelerometer
  //
  // Configure ADC pins. Enable A/D channel inputs
  //
  ACC_PORT_SEL0 |= ACC_X_PIN + ACC_Y_PIN + ACC_Z_PIN;    
  ACC_PORT_SEL1 |= ACC_X_PIN + ACC_Y_PIN + ACC_Z_PIN;
  ACC_PORT_DIR &= ~(ACC_X_PIN + ACC_Y_PIN + ACC_Z_PIN);
  
  // Enable ACC_POWER.  Actually enables thermistor power too.
  ACC_PWR_PORT_DIR |= ACC_PWR_PIN;              
  ACC_PWR_PORT_OUT |= ACC_PWR_PIN;
  
  // Configure ADC
  //Enable A/D channel inputs
  THERM_PORT_SEL0 |= THERM_IN_PIN;    
  THERM_PORT_SEL1 |= THERM_IN_PIN;
  THERM_PORT_DIR &= ~(THERM_IN_PIN);
  
  // Configure ADC10
  // 16ADCclks, MSC, ADC ON
  ADC10CTL0 = ADC10SHT_2 + ADC10MSC + ADC10ON;
  // sampling timer, s/w trig.,single sequence
  ADC10CTL1 = ADC10SHP + ADC10CONSEQ_1;
   // 16-bit resolution
  ADC10CTL2 |= ADC10RES;
  // A0,A1....A14(EoS), AVCC reference
  ADC10MCTL0 = ADC10INCH_14;            
  
  // Configure DMA0 (ADC10IFG trigger)
  // ADC10IFG trigger
  DMACTL0 = DMA0TSEL__ADC10IFG;             
  __data16_write_addr((unsigned short) &DMA0SA,(unsigned long) &ADC10MEM0);
  // Source single address  
  __data16_write_addr((unsigned short) &DMA0DA,(unsigned long) &ADC_Result[0]);
  // Destination array address  
  // 15 conversions 
  DMA0SZ = SIZE_OF_SEQUENCE;                
  DMA0CTL = DMADT_4 + DMADSTINCR_3 + DMAEN + DMAIE; 
}

//*****************************************************************************
//
//! TakeADCMeas
//!
//! \param  none
//!
//! \return none
//!
//! \brief  Take ADC measurement
//!				
//
//*****************************************************************************
void TakeAdcMeas(void)
{  
  while (ADC10CTL1 & BUSY); 
  ADC10CTL0 |= ADC10ENC | ADC10SC ;       // Start conversion 
  __bis_SR_register(CPUOFF + GIE);        // LPM0, ADC10_ISR will force exit  
  __no_operation();                       // For debug only
}

//*****************************************************************************
//
//! AdcUpdate
//!
//! \param  none
//!
//! \return none
//!
//! \brief  Takes a Temperature and Accel XYZ Measurement and stores them.
//!				
//
//*****************************************************************************
void AdcUpdate(void)
{
  TakeAdcMeas();
  // Update the Temperature
  AdcDevices.Temperature = ADC_Result[THERM_IN_IDX];
  AdcDevices.AccelX = ADC_Result[ACC_X_IDX];
  AdcDevices.AccelY = ADC_Result[ACC_Y_IDX];
  AdcDevices.AccelZ = ADC_Result[ACC_Z_IDX];
}

#pragma vector=DMA_VECTOR
__interrupt void DMA0_ISR (void)
{
  switch(__even_in_range(DMAIV,16))
  {
    case  0: break;                          // No interrupt
    case  2: 
      // sequence of conversions complete
      __bic_SR_register_on_exit(CPUOFF);     // exit LPM
      break;                                 // DMA0IFG
    case  4: break;                          // DMA1IFG
    case  6: break;                          // DMA2IFG
    case  8: break;                          // Reserved
    case 10: break;                          // Reserved
    case 12: break;                          // Reserved
    case 14: break;                          // Reserved
    case 16: break;                          // Reserved
    default: break; 
  }   
}
